#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import argparse
from pip._internal.req import parse_requirements
from pip._internal.download import PipSession, PipXmlrpcTransport
from pip._internal.models.index import PyPI
from pip._internal.commands.search import transform_hits, highest_version
from pip._vendor import pkg_resources
from pip._vendor.six.moves import xmlrpc_client


def load_requirements(files, session=False):
    requires = []
    for file in files:
        requires += list(parse_requirements(file, session=session))
    return dict([[r.name, r] for r in requires])


def search_pypi(queries, session=None):
    index_url = PyPI.pypi_url
    transport = PipXmlrpcTransport(index_url, session)
    pypi = xmlrpc_client.ServerProxy(index_url, transport)
    results = {}
    for query in queries:
        pypi_hits = pypi.search({'name': query, 'summary': query}, 'or')
        hits = transform_hits(pypi_hits)
        for h in hits:
            results[h['name']] = h
    return results


def print_results(queries, requires, hits):
    installed_packages = [p.project_name for p in pkg_resources.working_set]
    queries = sorted(queries, key=lambda x: x[0].lower())
    line = '{0:<24} {1:<40} {2:<12} {3}'
    print(line.format('name', 'requirements', 'installed', 'latest'))
    print(line.format('----', '------------', '---------', '------'))
    for q in queries:
        r = requires.get(q[0])
        hit = hits.get(q[0]) or hits.get(q[1])
        latest = highest_version(hit.get('versions', ['-'])) if hit else '-'
        if r:
            installed = r.installed_version
            name = r.name
            req = str(r.req)
        else:
            installed = None
            name = q[0]
            req = '-'
        if not installed:
            if name in installed_packages:
                dist = pkg_resources.get_distribution(name)
                installed = dist.version
            installed = installed or '-'
        print(line.format(name, req, installed, latest))


def main():
    parser = argparse.ArgumentParser(description='check pip packages version')
    parser.add_argument('-r', '--requirement', dest='requirements', default=[], action='append')
    parser.add_argument('specifiers', nargs='*')
    options = parser.parse_args()
    if not options.requirements and not options.specifiers:
        parser.print_help()
        return
    session = PipSession()
    requires = load_requirements(options.requirements, session=session)
    queries = [[r.name, r.req.name] for r in requires.values()] + [[s, s] for s in options.specifiers]
    hits = search_pypi(queries, session=session)
    print_results(queries, requires, hits)


if __name__ == '__main__':
    main()
